/***************************************************************************
*	NAME:  UTIL0.C
**	COPYRIGHT:
**	"Copyright (c) 1992, by FORTE
**
**       "This software is furnished under a license and may be used,
**       copied, or disclosed only in accordance with the terms of such
**       license and with the inclusion of the above copyright notice.
**       This software or any other copies thereof may not be provided or
**       otherwise made available to any other person. No title to and
**       ownership of the software is hereby transfered."
****************************************************************************
*  CREATION DATE: 11/18/92
*--------------------------------------------------------------------------*
*     VERSION	DATE	   NAME		DESCRIPTION
*>	1.0	11/18/92		Original
***************************************************************************/
#include <dos.h>
#include <conio.h>

#include "forte.h"
#include "gf1proto.h"
#include "gf1hware.h"
#include "gf1os.h"

extern ULTRA_DATA _gf1_data;

int
UltraGetOutput(void)
{
return (!(_gf1_data.mix_image & ENABLE_OUTPUT));
}

void
UltraEnableOutput(void)
{
_gf1_data.mix_image &= ~ENABLE_OUTPUT;
outp(_gf1_data.mix_control,_gf1_data.mix_image);
}

void
UltraDisableOutput(void)
{
_gf1_data.mix_image |= ENABLE_OUTPUT;
outp(_gf1_data.mix_control,_gf1_data.mix_image);
}

int
UltraGetLineIn(void)
{
return(!(_gf1_data.mix_image & ENABLE_LINE_IN));
}

void
UltraEnableLineIn(void)
{
_gf1_data.mix_image &= ~ENABLE_LINE_IN;
outp(_gf1_data.mix_control,_gf1_data.mix_image);
}

void
UltraDisableLineIn(void)
{
_gf1_data.mix_image |= ENABLE_LINE_IN;
outp(_gf1_data.mix_control,_gf1_data.mix_image);
}

int
UltraGetMicIn(void)
{
return(_gf1_data.mix_image & ENABLE_MIC_IN);
}

void
UltraEnableMicIn(void)
{
_gf1_data.mix_image |= ENABLE_MIC_IN;
outp(_gf1_data.mix_control,_gf1_data.mix_image);
}

void
UltraDisableMicIn(void)
{
_gf1_data.mix_image &= ~ENABLE_MIC_IN;
outp(_gf1_data.mix_control,_gf1_data.mix_image);
}

